#!/usr/bin/env python

import sys
import base64
from farcaster.fcproto.message_pb2 import MessageType, Message
from google.protobuf.json_format import ParseDict
import argparse
import json
from . __about__ import version

def json2fario():
	parser = argparse.ArgumentParser(prog="json2fario", description="Json to fario. Expects an array of Message in json format.")
	parser.add_argument('--version', action='version', version='%(prog)s v'+version)
	args = parser.parse_args()

	j = json.loads(sys.stdin.read())
	for msg in j:
		pb = ParseDict(msg, Message()) 
		out = base64.b64encode(pb.SerializeToString()).decode('ascii')
		print(out)
		
if __name__ == '__main__':
	json2fario()


#!/usr/bin/env python
import os
import sys
from datetime import datetime
#from dotenv import load_dotenv
from eth_account import Account
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import SignatureScheme, HashScheme, Embed
from farcaster.fcproto.onchain_event_pb2 import OnChainEventType, OnChainEvent
from farcaster.fcproto.request_response_pb2 import StoreType
from farcaster import FARCASTER_EPOCH
from farcaster import Message
from . __about__ import version
from . config import get_conf, check_conf
from web3 import Web3

import argparse

CONF = {}

def get_hub_address(args) -> str:
    check_conf(CONF, ['hub'])
    hub_address = CONF['hub']
    return hub_address

def get_usage(fid, args):
    def count(c, fn, args):
        ret = fn(**args)
        if hasattr(ret,'messages'):
            c = c + len(ret.messages)
        if hasattr(ret,'proofs'):
            c = c + len(ret.proofs)
        if getattr(ret,'next_page_token', None):
            args['page_token'] = ret.next_page_token
            return( count(c, fn, args) )
        else:
            return c
    hub_address = get_hub_address(args)
    hub = HubService(hub_address, use_async=False, use_ssl=CONF['ssl'])
    usage = (
        ('casts', hub.GetCastsByFid, {"fid":fid, "page_size":1000} ),
        ('links', hub.GetLinksByFid, {"fid":fid, "page_size":1000} ),
        ('likes', hub.GetReactionsByFid, {"fid":fid, "reaction_type":1, "page_size":1000} ),
        ('recasts', hub.GetReactionsByFid, {"fid":fid, "reaction_type":2, "page_size":1000} ),
        ('user_data', hub.GetUserDataByFid, {"fid":fid, "page_size":1000} ),
        ('proofs', hub.GetUserNameProofsByFid, {"fid":fid, "page_size":1000} ),
        ('verifications', hub.GetVerificationsByFid, {"fid":fid, "page_size":1000} )
        )
    usage_counts = {}
    for u in usage:
        usage_counts[u[0]] = count(0, u[1], u[2])
    return usage_counts

def get_names_by_fid(args, fid):
    hub_address = get_hub_address(args)
    hub = HubService(hub_address, use_async=False, use_ssl=CONF['ssl'])
    ret  = hub.GetUserNameProofsByFid(fid, page_size=None)
    return [ (m.name.decode('ascii'), '0x'+m.owner.hex()) for m in ret.proofs ]

def get_addr_by_fid(args, fid):
    hub_address = get_hub_address(args)
    hub = HubService(hub_address, use_async=False, use_ssl=CONF['ssl'])
    ret  = hub.GetIdRegistryOnChainEvent(fid)
    return (
        ret.id_register_event_body.to.hex(), 
        ret.id_register_event_body.recovery_address.hex()
    )
def get_storage_rent_by_fid(args, fid):
    storage=[]
    hub_address = get_hub_address(args)
    hub = HubService(hub_address, use_async=False, use_ssl=CONF['ssl'])
    ret  = hub.GetOnChainEvents(fid=fid, event_type=4, page_size=None)
    return [
        (e.storage_rent_event_body.units,
        e.storage_rent_event_body.expiry,
        e.storage_rent_event_body.payer.hex())
        for e in ret.events
    ]
    
def get_storage_limits_by_fid(args, fid):
    hub_address = get_hub_address(args)
    hub = HubService(hub_address, use_async=False, use_ssl=CONF['ssl'])
    ret  = hub.GetCurrentStorageLimitsByFid(fid)
    limits = { StoreType.Name(l.store_type)[11:].lower(): l.limit for l in ret.limits }
    return (limits)

def get_op_balance_by_addr(args, addr):
    check_conf(CONF, ['op_eth_provider'])
    w3 = Web3(Web3.HTTPProvider(CONF['op_eth_provider']))
    balance = w3.eth.get_balance(addr)
    return balance
    
def print_account(args, account):
    def _print(key, val, rawdata=False):
        if rawdata:
            print(val, end="\t")
        else:
            print(key.ljust(15),val)
    if args.fid:
        _print('fid',account['fid'], args.raw)
    if args.addr:
        _print('address', '0x'+account['addr'], args.raw)
        if args.balance:
            _print('balance', str(account['balance']/(10**18)) + ' OP ETH', args.raw)
    if args.recovery:
        _print('recovery', '0x'+account['recovery'], args.raw)
    if args.fname:
        fname = [ n for n in account['names'] if n[0][-4:] !='.eth' ]
        if len(fname)>0:
            _print('fname', fname[0][0], args.raw )
        else:
            _print("fname", '-', args.raw)
    if args.name:
        name = [ n for n in account['names'] if n[0][-4:] =='.eth' ]
        if len(name)>0:
            _print('name', name[0][0], args.raw )
        else:
            _print("name", '-', args.raw)
    if args.storage_rent:
        if not args.raw:
            print("\nStorage Units Rented")
            print("====================")
            _print("Units", "Date                Paid by", False)
        for s in account['storage_rent']:
            timestamp = datetime.fromtimestamp(s[1]+FARCASTER_EPOCH).strftime('%Y-%m-%dT%H:%M:%S')
            _print(str(s[0]), f"{timestamp} 0x{s[2]}", args.raw)
    if args.storage_limits:
        if not args.raw:
            print("\nCurrent Storage Limits")
            print("======================")
        for l in account['storage_limits']:
            _print(l, account['storage_limits'][l], args.raw)
    if args.storage_usage:
        if not args.raw:
            print("\nCurrent Usage")
            print("=============")
        for l in account['storage_usage']:
            _print(l, account['storage_usage'][l], args.raw)
    print()


def by_fid(args, fid=None):
    account={}
    account['fid'] = fid if fid else args.account_fid

    if args.all:
        args.addr = True
        args.fname = True
        args.storage_rent = True
        args.storage_limits = True 
        args.storage_usage = True 

    if args.addr or args.recovery:
        account['addr'], account['recovery'] = get_addr_by_fid(args, account['fid'])
        if args.balance:
            account['balance'] = get_op_balance_by_addr(args, Web3.to_checksum_address('0x'+account['addr']))
    if args.fname or args.name:
        account['names'] = get_names_by_fid(args, account['fid'])
    if args.storage_rent:
        account['storage_rent']=get_storage_rent_by_fid(args, account['fid'])
    if args.storage_limits:
        account['storage_limits'] = get_storage_limits_by_fid(args, account['fid'])
    if args.storage_usage:
        account['storage_usage'] = get_usage(account['fid'], args)
    print_account(args, account)

def by_name(args):
    hub_address = get_hub_address(args)
    hub = HubService(hub_address, use_async=False, use_ssl=CONF['ssl'])
    ret  = hub.GetUsernameProof(args.name)
    fid = ret.fid
    by_fid(args, fid)

def from_secret(args):
    print(  "=== DANGER!!! ===")
    print(  "Revealing your secret phrase (seed or mnemonic) to someone, gives\n"
            "them full control over your Farcaster account, including the ability\n"
            "to move your fid to an other wallet!\n\n"
            "This script will calculate your privet key using your secret phrase.\n"
            "This script runs locally and does not send ANY data to any third party.\n\n"
            "Do your own research before trusting anyone with your account's key\n"
            "or mnemonic. This includes this script too.\n\n"
    )
    seed = input("Enter secret phrase: ")
    if seed.strip():
        Account.enable_unaudited_hdwallet_features()
        acc = Account.from_mnemonic(seed)
        print("Account key:".ljust(15), acc.key.hex())
        print("Account addr:".ljust(15), f"0x{acc.address}")
    
def main():
    parser = argparse.ArgumentParser(prog='fario-account')
    parser.add_argument('--version', action='version', version='%(prog)s v'+version)
    parser.add_argument('--raw', action='store_true', help="Output raw values, tab separated.")
    parser.add_argument("--hub", help="Use the hub at <HUB>. Ex. --hub 192.168.1.1:2283", type=str)
    parser.add_argument("--ssl", help="Use SSL", action="store_true")
    parser.add_argument("--fid", action='store_true', help="Print fid")
    parser.add_argument("--fname", action='store_true', help="Print fname")
    parser.add_argument("--name", action='store_true', help="Print name")
    parser.add_argument("--addr", action='store_true', help="Print custody address")
    parser.add_argument("--balance", action='store_true', help="Print custody address balance")
    parser.add_argument("--storage-rent", action='store_true', help="Print storage rent events")
    parser.add_argument("--storage-limits", action='store_true', help="Print storage limits")
    parser.add_argument("--storage-usage", action='store_true', help="Print storage usage")
    parser.add_argument("--recovery", action='store_true', help="Print recovery address")
    parser.add_argument("--all", action='store_true', help="Print all available information")
    subparser = parser.add_subparsers()

    cmd_addr_by_fid = subparser.add_parser("byfid", description="Get account info using the fid")
    cmd_addr_by_fid.add_argument("account_fid", type=int, help="User's fid.")
    cmd_addr_by_fid.set_defaults(func=by_fid)

    cmd_addr_by_name = subparser.add_parser("byname", description="Get account info using name or fname")
    cmd_addr_by_name.add_argument("name", type=str, help="User's name.")
    cmd_addr_by_name.set_defaults(func=by_name)

    cmd_addr_from_secret = subparser.add_parser("fromsecret", description="Get account details using the secret seed phrase.")
    cmd_addr_from_secret.set_defaults(func=from_secret)

    args = parser.parse_args()

    CONF.update(get_conf(args=args))

    if 'func' in args:
        args.func(args)
    

if __name__ == '__main__':
    main()


#!/usr/bin/env python
from farcaster.HubService import HubService
from . config import get_conf
from . __about__ import version

import argparse

def get_hub_info():
	parser = argparse.ArgumentParser(prog="fario-hubinfo", description="Show a hub's information.")
	parser.add_argument('--version', action='version', version='%(prog)s v'+version)
	parser.add_argument("--hub", help="Use the hub at <ADDRESS>. Ex. --hub 192.168.1.1:2283", type=str)
	parser.add_argument("--ssl", help="Use SSL", action="store_true")
	args = parser.parse_args()

	conf = get_conf(required=['hub'], args=args)

	hub = HubService(conf['hub'], use_async=False, use_ssl=conf['ssl'])
	ret  = hub.GetInfo()
	print('address:', conf['hub'])
	print(ret)

if __name__ == '__main__':
	get_hub_info()


#!/usr/bin/env python

import argparse
import sys

from __about__ import version
def call_fario_out(args):
    print(args)

def call_fario_in(args):
    import fario_in
    fario_in.main(args)

def main():
    parser = argparse.ArgumentParser(prog='fario')
    parser.add_argument('--version', action='version', version='%(prog)s v'+version)
    subparser = parser.add_subparsers()

    fario_out = subparser.add_parser("out", description="Export Farcaster data.")
    fario_out.add_argument("fid", type=int, help="FID")
    fario_out.add_argument("--casts", help="User casts", action="store_true")
    fario_out.add_argument("--links", help="User links", action="store_true")
    fario_out.add_argument("--recasts", help="User recasts", action="store_true")
    fario_out.add_argument("--likes", help="User likes", action="store_true")
    fario_out.add_argument("--inlinks", help="Inbound links for user", action="store_true")
    fario_out.add_argument("--profile", help="User profile data", action="store_true")
    fario_out.add_argument("--all", help="Equivalent to --casts --links --recasts --likes --profile", action="store_true")
    fario_out.add_argument("--limit", type=int, help="Number of records. If more than one types of data are exported, the limit applies to each one separately.", default=sys.maxsize)
    fario_out.add_argument("--hub", help="Use the hub at <ADDRESS>. Ex. --hub 192.168.1.1:2283", type=str)
    fario_out.add_argument("--wait", type=int, help="Wait for <WAIT> milliseconds between reads.", default=0)

    fario_in = subparser.add_parser("in")
    fario_in.add_argument("fid", type=int, help="FID")
    fario_in.set_defaults(func=call_fario_in)

    args = parser.parse_args()
    args.func(args)

if __name__ == '__main__':
    main()


#!/usr/bin/env python
import sys
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import SignatureScheme, HashScheme, Embed
from farcaster import Message, FARCASTER_EPOCH
import argparse
from . config import get_conf
from . __about__ import version

def main():
	parser = argparse.ArgumentParser(prog="fario-cast", description="Send a cast from the command line.")
	parser.add_argument('--version', action='version', version='%(prog)s v'+version)
	parser.add_argument("--hub", help="Use the hub at <ADDRESS>. Ex. --hub 192.168.1.1:2283", type=str)
	parser.add_argument("--ssl", help="Use SSL", action="store_true")
	parser.add_argument("--signer", type=str, help="Signer's private key")
	parser.add_argument("--user-fid", type=int, help="User's fid")
	parser.add_argument("--timestamp", type=int, help="Cast date (unix timestamp)")
	parser.add_argument("--embed", action="append", help="Embed a URL. Can be called multiple times.")
	parser.add_argument("-v", "--verbose", help="increase output verbosity", action="store_true")
	parser.add_argument("text", type=str, help="Cast text")
	args = parser.parse_args()
	
	conf=get_conf(required=['hub', 'user_fid', 'signer'], args=args)
	user_fid = conf['user_fid']

	timestamp = args.timestamp - FARCASTER_EPOCH if args.timestamp else None
	if args.timestamp and timestamp < 0:
		print("Error: Timestamp must be after Jan 1, 2021 00:00:00 UTC.")
		sys.exit(1)

	if args.embed and len(args.embed) > 2:
		print("Error: More than 2 embeds.")
		sys.exit(1)
	hub = HubService(conf['hub'], use_async=False, use_ssl=conf['ssl'])

	message_builder = Message.MessageBuilder(
		HashScheme.HASH_SCHEME_BLAKE3, 
		SignatureScheme.SIGNATURE_SCHEME_ED25519, 
		bytes.fromhex(conf['signer'][2:])
	)
	
	if args.embed:
		data = message_builder.cast.add(
			fid = int(user_fid), 
			text = args.text[:320],
			embeds = [ Embed(url=e) for e in args.embed ],
			timestamp = timestamp
		)
	else:
		data = message_builder.cast.add(
			fid = int(user_fid), 
			text = args.text[:320],
			timestamp = timestamp
		)
	msg  = message_builder.message(data)

	ret  = hub.SubmitMessage(msg)
	if args.verbose:
		print(f"Message posted: {args.text[:320]}")

if __name__ == '__main__':
	main()


#!/usr/bin/env python

import os
import sys
import datetime
import base64
from farcaster.Signer import Signer, removeSigner
from nacl.signing import SigningKey
from blake3 import blake3
from farcaster.fcproto.message_pb2 import HashScheme, Message, MessageData
from . config import get_conf

import argparse
from . __about__ import version

def signer_add(args):
    conf = get_conf(
        required = ['op_eth_provider', 'user_fid', 'user_key', 'app_fid', 'app_key'],
        args = args
        )
    s = Signer( conf['op_eth_provider'], int(conf['user_fid']), conf['user_key'], int(conf['app_fid']), conf['app_key'] )

    tx_hash = s.approve_signer()
    signer_private_key = s.key
    signer_public_key = s.signer_pub()

    if not args.raw:
        print(f"=== New Signer approved =====================")
        print(f"Tx: {tx_hash}")
        print(f"User Fid: {conf['user_fid']}")
        print(f"App Fid: {conf['app_fid']}")
        print(f"Signer public key: 0x{s.signer_pub().hex()}")
        print(f"Signer private key: {s.key.hex()}")
        print(f"=== MAKE SURE YOU SAVE THE PRIVATE KEY!!! ===")
    else:
        print(f"{tx_hash}\t{conf['user_fid']}\t{conf['app_fid']}\t0x{s.signer_pub().hex()}\t{s.key.hex()}" )

def signer_revoke(args):
    conf = get_conf(
        required = ['op_eth_provider', 'user_key'],
        args = args
        )

    provider = conf['op_eth_provider']
    user_key = conf['user_key']

    tx = removeSigner(provider, user_key, args.signer)
    if not args.raw:
        print(f"Signer {args.signer} removed. Tx={tx}")
    else:
        print(tx)

def signer_list(args):
    conf = get_conf(
        required = ['hub'],
        args = args
        )
    hub_address = conf['hub']

    from farcaster.HubService import HubService
    hub = HubService(hub_address, use_async=False, use_ssl=conf['ssl'])
    r = hub.GetOnChainSignersByFid(args.fid)
    signers = []
    fid_to_name = {}
    for evt in r.events:
        signers.append({
            "pub_key": f"0x{evt.signer_event_body.key.hex()}",
            "timestamp": evt.block_timestamp,
            "signer_fid": int(evt.signer_event_body.metadata.hex()[64:128], 16)
            })
    for s in sorted(signers, key=lambda d: d['timestamp']):
        if args.with_fnames:
            if s['signer_fid'] not in fid_to_name:
                ret = hub.GetUserNameProofsByFid(s['signer_fid'])
                if ret.proofs:
                    name = "@" + [ proof.name for proof in ret.proofs if proof.type==1][0].decode('ascii')
                else:
                    name = f"{s['signer_fid']} (no name found)"
                fid_to_name[s['signer_fid']] = name
            print(f"{s['pub_key']}\t{s['timestamp']}\t{s['signer_fid']}\t{fid_to_name[s['signer_fid']]}")
        else:
            print(f"{s['pub_key']}\t{s['timestamp']}\t{s['signer_fid']}")

def signer_sign(args):
    conf = get_conf(
        required = ['signer'],
        args = args
    )

    signer=SigningKey(bytes.fromhex(conf['signer'][2:]))
    signer_pub_key=signer.verify_key.encode()

    for line in sys.stdin:
        m = Message.FromString(base64.b64decode(line))
        if m.hash_scheme == HashScheme.HASH_SCHEME_BLAKE3:
            data_serialized = m.data.SerializeToString()
            msg_hash = blake3(data_serialized).digest(length=20)
            msg_signature = signer.sign(msg_hash).signature
            m.signer=signer_pub_key
            m.signature=msg_signature
            if msg_hash != m.hash:
                if args.keep_hash == True:
                    print("Error: [fario-signers] New message hash != old message hash, but --keep-hash=True.",
                        file=sys.stderr)
                    sys.exit(1)
                else:
                    m.hash = msg_hash
                    m.data_bytes = data_serialized
            out = base64.b64encode(m.SerializeToString()).decode('ascii')
            print(out)
        else:
            print("Error: [fario-signers] HashScheme={m.hash_scheme} not supported.", file=sys.stderr)
            sys.exit(1)

def main():
    parser = argparse.ArgumentParser(prog='fario-signers')
    parser.add_argument('--version', action='version', version='%(prog)s v'+version)
    parser.add_argument('--raw', action='store_true', help="Output raw values, tab separated.")
    subparser = parser.add_subparsers()

    cmd_signer_add = subparser.add_parser("add", description="""Create a new signer.
        Using --raw will output tx_hash, user_fis, app_fid, signer_bublic_key,
        signer_private_key as a tab-separated list.
        """)
    cmd_signer_add.add_argument("--provider", help="OP Eth provider endpoint")
    cmd_signer_add.add_argument("--user-fid", help="User's fid.")
    cmd_signer_add.add_argument("--user-key", help="User's private key in hex.")
    cmd_signer_add.add_argument("--app-fid", help="Application's fid.")
    cmd_signer_add.add_argument("--app-key", help="Application's private key in hex.")
    cmd_signer_add.set_defaults(func=signer_add)

    cmd_signer_remove = subparser.add_parser("remove", description="""Remove a signer.
        Using --raw will output only the tx_hash.""")
    cmd_signer_remove.add_argument("--op_eth_provider", help="OP Eth provider endpoint")
    cmd_signer_remove.add_argument("--user-key", help="User's private key in hex.")
    cmd_signer_remove.add_argument("signer", help="Signer's public key in hex.")
    cmd_signer_remove.set_defaults(func=signer_revoke)

    cmd_signer_list = subparser.add_parser("list", description="list signers")
    cmd_signer_list.add_argument("--hub", help="Use the hub at <HUB>. Ex. --hub 192.168.1.1:2283", type=str)
    cmd_signer_list.add_argument("--ssl", help="Use SSL", action="store_true")
    cmd_signer_list.add_argument('--with-fnames', action='store_true', help="Display the fname of the Signer owner.")
    cmd_signer_list.add_argument("fid", type=int, help="Signers for <FID>")
    cmd_signer_list.set_defaults(func=signer_list)

    cmd_signer_sign = subparser.add_parser("sign", 
        description="Read messages and sign (or re-sign) using a new signer.\n"
        "Reads from stdin, writes to stdout. In and out are in fario format.")
    cmd_signer_sign.add_argument("--signer", type=str, help="Signer's private key")
    cmd_signer_sign.add_argument('--keep-hash', default="false", action='store_true', help="Do not change the message hash.")
    cmd_signer_sign.set_defaults(func=signer_sign)    

    args = parser.parse_args()

    if 'func' in args:
        args.func(args)
    

if __name__ == '__main__':
    main()



# SPDX-FileCopyrightText: 2024-present Panayotis Vryonis <vrypan@gmail.comm>
#
# SPDX-License-Identifier: MIT
import os
import click
import time
from dotenv import load_dotenv

from . __about__ import version
from . config import get_conf
from farcaster import Message, FARCASTER_EPOCH
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import SignatureScheme, HashScheme, UserDataType

USER_DATA_TYPES = {
    'none': UserDataType.USER_DATA_TYPE_NONE,
    'pfp': UserDataType.USER_DATA_TYPE_PFP,
    'display': UserDataType.USER_DATA_TYPE_DISPLAY,
    'bio': UserDataType.USER_DATA_TYPE_BIO,
    'url': UserDataType.USER_DATA_TYPE_URL,
    'username': UserDataType.USER_DATA_TYPE_USERNAME
}

@click.group(context_settings={"help_option_names": ["-h", "--help"]}, invoke_without_command=True)
@click.version_option(version=version, prog_name="fario-userdata")
def fario_user_data():
    """Farcaster Network Mapper
    """
    CONFIG_FILE = os.path.expanduser('~/.fario')
    load_dotenv(CONFIG_FILE)
    load_dotenv()
    

@fario_user_data.command()
@click.option('--hub', default='hoyt.farcaster.xyz:2283', help="Use the hub at <ADDRESS:PORT>. Ex. --hub 192.168.1.1:2283", envvar='hub')
@click.option('--ssl', is_flag=True, default=False, help="Use SSL")
@click.option('--signer', help="Signer's private key", required=True, envvar='signer')
@click.argument('user-fid', type=click.INT)
@click.argument('data-type', type=click.Choice([k for k in USER_DATA_TYPES.keys()], case_sensitive=False), required=True)
@click.argument('data-value')
def set(hub, ssl, signer, user_fid, data_type, data_value):

    timestamp = int(time.time())-FARCASTER_EPOCH
    hub = HubService(hub, use_async=False, use_ssl=ssl)

    message_builder = Message.MessageBuilder(
        HashScheme.HASH_SCHEME_BLAKE3, 
        SignatureScheme.SIGNATURE_SCHEME_ED25519, 
        bytes.fromhex(signer[2:])
    )
    
    data = message_builder.user_data.add(
        fid = user_fid, 
        data_type = USER_DATA_TYPES[data_type],
        data_value = data_value
    )

    msg  = message_builder.message(data)

    ret  = hub.SubmitMessage(msg)
    
    print(f"USER_DATA_TYPE_{data_type.upper()} set to {data_value}")


@fario_user_data.command()
@click.option('--hub', default='hoyt.farcaster.xyz:2283', help="Use the hub at <ADDRESS:PORT>. Ex. --hub 192.168.1.1:2283", envvar='hub')
@click.option('--ssl', is_flag=True, default=False, help="Use SSL")
@click.argument('user-fid', type=click.INT)
@click.argument('data-type', type=click.Choice([k for k in USER_DATA_TYPES.keys()], case_sensitive=False), required=True)
def get(hub, ssl, user_fid, data_type):
    hub = HubService(hub, use_async=False, use_ssl=ssl)
    response = hub.GetUserDataByFid(user_fid, 100)
    for c in response.messages:
        if c.data.user_data_body.type == USER_DATA_TYPES[data_type]:
            print(c.data.user_data_body.value)







#!/usr/bin/env python
import os
import sys
from time import sleep
from dotenv import load_dotenv
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import MessageType, Message
import base64
import argparse
from . config import get_conf
from . __about__ import version

def get_data(method, fid, page_size, limit, wait):
	first_run=True
	page_token=None
	count = 0
	while (page_token or first_run) and count<limit:
		first_run=False
		casts = method(fid, page_size=100, page_token=page_token)
		for cast in casts.messages:
			out = base64.b64encode(cast.SerializeToString()).decode('ascii')
			yield out
			count +=1
			if count==limit:
				break
		page_token = casts.next_page_token
		sleep(wait)

def get_reactions(method, fid, reaction_type, page_size, limit, wait):
	first_run=True
	page_token=None
	count = 0
	while (page_token or first_run) and count<limit:
		first_run=False
		casts = method(fid=fid, reaction_type=reaction_type, page_size=100, page_token=page_token)
		for cast in casts.messages:
			out = base64.b64encode(cast.SerializeToString()).decode('ascii')
			yield out
			count +=1
			if count==limit:
				break
		page_token = casts.next_page_token
		sleep(wait)

def main():		
	parser = argparse.ArgumentParser(prog="fario-out", description="Export Farcaster data.")
	parser.add_argument('--version', action='version', version='%(prog)s v'+version)
	parser.add_argument("fid", type=int, help="Export messages from fid=FID")
	parser.add_argument("--casts", help="User casts", action="store_true")
	parser.add_argument("--links", help="User links", action="store_true")
	parser.add_argument("--recasts", help="User recasts", action="store_true")
	parser.add_argument("--likes", help="User likes", action="store_true")
	parser.add_argument("--inlinks", help="Inbound links for user", action="store_true")
	parser.add_argument("--profile", help="User profile data", action="store_true")
	parser.add_argument("--all", help="Equivalent to --casts --links --recasts --likes --profile", action="store_true")
	parser.add_argument("--limit", type=int, help="Number of records. If more than one types of data are exported, the limit applies to each one separately.", default=sys.maxsize)
	parser.add_argument("--hub", help="Use the hub at <HUB>. Ex. --hub 192.168.1.1:2283", type=str)
	parser.add_argument("--ssl", help="Use SSL", action="store_true")
	parser.add_argument("--wait", type=int, help="Wait for <WAIT> milliseconds between reads.", default=0)
	args = parser.parse_args()

	conf = get_conf(required=['hub'], args=args)

	hub = HubService(conf['hub'], use_async=False, use_ssl=conf['ssl'])

	if args.casts or args.all:
		for c in get_data(hub.GetCastsByFid, args.fid, 100, args.limit, args.wait):
			print(c)
	if args.links or args.all:
		for c in get_data(hub.GetLinksByFid, args.fid, 100, args.limit, args.wait):
			print(c)
	if args.likes or args.all:
		for c in get_reactions(hub.GetReactionsByFid, args.fid, 1, 100, args.limit, args.wait):
			print(c)
	if args.recasts or args.all:
		for c in get_reactions(hub.GetReactionsByFid, args.fid, 2, 100, args.limit, args.wait):
			print(c)
	if args.profile or args.all:
		for c in get_data(hub.GetUserDataByFid, args.fid, 100, args.limit, args.wait):
			print(c)
	if args.inlinks:
		for c in get_data(hub.GetLinksByTarget, args.fid, 100, args.limit, args.wait):
			print(c)
	

if __name__ == '__main__':
	main()


#!/usr/bin/env python
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import SignatureScheme, HashScheme, Embed
from farcaster import Message
from . config import get_conf
from . __about__ import version

import argparse

def main():
	parser = argparse.ArgumentParser(prog="fario-fid-byname", description="Get a user's fid")
	parser.add_argument('--version', action='version', version='%(prog)s v'+version)
	parser.add_argument("--hub", help="Use the hub at <ADDRESS>. Ex. --hub 192.168.1.1:2283", type=str)
	parser.add_argument("--ssl", help="Use SSL", action="store_true")
	parser.add_argument("-v", "--verbose", help="increase output verbosity", action="store_true")
	parser.add_argument("username", type=str, help="Cast text")
	args = parser.parse_args()

	conf = get_conf(required=['hub'], args=args)

	hub = HubService(conf['hub'], use_async=False, use_ssl=conf['ssl'])
	ret  = hub.GetUsernameProof(args.username)

	if args.verbose:
		print(f"User {args.username} is fid {ret.fid}")
	else:
		print(ret.fid)


if __name__ == '__main__':
	main()


#!/usr/bin/env python
import os
import sys
from time import sleep
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import Message
import base64
import argparse
from . config import get_conf
from . __about__ import version

def main():
	parser = argparse.ArgumentParser(prog="fario-in", description="Send messages to Farcaster hub.")
	parser.add_argument('--version', action='version', version='%(prog)s v'+version)
	parser.add_argument("--hub", help="Use the hub at <HUB>. Ex. --hub 192.168.1.1:2283", type=str)
	parser.add_argument("--ssl", help="Use SSL", action="store_true")
	parser.add_argument("--wait", type=int, help="Wait for <WAIT> milliseconds between message submissions.", default=100)
	args = parser.parse_args()

	conf = get_conf(required=['hub'], args=args)

	hub = HubService(conf['hub'], use_async=False, use_ssl=conf['ssl'])

	for line in sys.stdin:
		m = Message.FromString(base64.b64decode(line))
		r = hub.SubmitMessage(m)
		sleep(args.wait/1000)

if __name__ == '__main__':
	main()


#!/usr/bin/env python
from farcaster.HubService import HubService
#from . config import get_conf
import os
import sys
import re
import argparse
from datetime import datetime
from dotenv import load_dotenv
import sqlite3
from termcolor import colored
import textwrap
from farcaster.fcproto.message_pb2 import Message, ReactionType
from farcaster import FARCASTER_EPOCH

def get_conf(required=[], args=None) -> str:

    CONFIG_FILE = os.path.expanduser('~/.fario')

    load_dotenv(CONFIG_FILE)
    load_dotenv()

    conf = {}
    params = ( 
        'hub', 'user_fid', 'user_key', 'app_fid', 'app_key', 'signer', 'op_eth_provider'
    )
    for p in params:
        conf[p] = os.getenv(p,'')
        if args and hasattr(args, p) and getattr(args, p):
            conf[p] = getattr(args,p)
    if len(required)>0 :
        check_conf(conf, required)
    return conf
def check_conf(conf, keys):
    for k in keys:
        if not conf[k]:
            print(f"Error: {k} is not set. Use the corresponding option, "
                    "an environment varaible, or set it in ~/.fario", 
                file=sys.stderr)
            sys.exit(1)

class Proxy:
    TTL = 60*5
    def __init__(self):
        self.conf = get_conf(required=['hub'])
        self.hub = HubService(self.conf['hub'], use_async=False)
        self.files_path = os.path.expanduser('~/.fcsh')
        if not os.path.isdir(self.files_path):
            os.mkdir(self.files_path)
        self.db_path = os.path.join(self.files_path, 'db')
        if not os.path.isfile(self.db_path):
            self.db_open()
            self.db_create()
        else:
            self.db_open()
        
    def db_open(self):
        self.db_conn = sqlite3.connect(self.db_path, isolation_level="IMMEDIATE")
        self.db_conn.row_factory = sqlite3.Row
        self.db_cur = self.db_conn.cursor()
        return self

    def db_close(self):
        self.db_conn.commit()
        self.db_conn.close()

    def db_create(self):
        self.db_cur.execute("""
            CREATE TABLE IF NOT EXISTS users(
                fid INTEGER,
                fname TEXT,
                name TEXT,
                upd_ts INTEGER,
                primary key(fid))
            """)
        self.db_cur.execute("CREATE UNIQUE INDEX fname_idx ON users(fname)")
        self.db_cur.execute("CREATE UNIQUE INDEX name_idx ON users(name)")

        self.db_cur.execute("""
            CREATE TABLE IF NOT EXISTS user_data(
                fid INTEGER,
                data_type INTEGER,
                value TEXT,
                upd_ts INTEGER,
                primary key(fid,data_type))
            """)

        self.db_cur.execute("""
            CREATE TABLE IF NOT EXISTS casts(
                fid INTEGER,
                hash TEXT,
                parent TEXT,
                msg BLOB,
                upd_ts INTEGER,
                primary key(fid,hash))
            """)
        self.db_cur.execute("CREATE INDEX cast_parent_idx ON casts(parent)")
        self.db_cur.execute("""
            CREATE TABLE IF NOT EXISTS reactions(
                fid INTEGER,
                hash TEXT,
                u_fid INTEGER,
                reaction_type INTEGER,
                ts INTEGER,
                upd_ts INTEGER,
                primary key(fid, hash, u_fid, reaction_type))
            """)
        self.db_cur.execute("CREATE INDEX reactions_ts_idx ON reactions(fid, hash, u_fid, reaction_type, ts)")
        self.db_conn.commit()

    def get_fid_by_name(self, name):
        ret  = self.hub.GetUsernameProof(name)
        if ret:
            return ret.fid
        else:
            print('Error: Name not found.', file=sys.stderr)
            sys.exit(1)

    def get_user_by_fid(self, fid):
        self.db_cur.execute("SELECT * FROM users WHERE fid=?", (fid,))
        r = self.db_cur.fetchone()
        if r:
            return r
        else:
            ret  = self.hub.GetUserNameProofsByFid(fid, page_size=None)
            proofs = [ m.name.decode('ascii') for m in ret.proofs ]
            fname = ''
            name = ''
            ts = int(datetime.now().timestamp())
            for n in proofs:
                if n[-4:] =='.eth':
                    name = n
                else:
                    fname = n
            self.db_cur.execute("INSERT OR REPLACE INTO users(fid, fname, name, upd_ts) VALUES(?,?,?,?)", (fid, fname, name, ts))
            self.db_conn.commit()
            return {'fid': fid, 'fname': fname, 'name': name }

    def get_username_by_fid(self, fid):
        user = self.get_user_by_fid(fid)
        if user['name']:
            return user['name']
        elif user['fname']:
            return user['fname']
        else:
            return str(fid)
    def hdb_get_user_data(self, fid, user_data_type):
        ts = int(datetime.now().timestamp())
        self.db_cur.execute("SELECT * FROM user_data WHERE fid=? AND data_type=? AND upd_ts>?", (fid,user_data_type,ts-(self.TTL*10)))
        r = self.db_cur.fetchone()
        if r:
            return(r['value'])
        else:
            msg = self.hub.GetUserData(fid, user_data_type)
            if not msg:
                value = None
            else:
                value = msg.data.user_data_body.value
            self.db_cur.execute(
                "REPLACE INTO user_data(fid,data_type,value,upd_ts) VALUES(?,?,?,?)",
                (fid, user_data_type, value, ts)
            )
            self.db_cur.commit()
            return value

    def pp_date(self, t: int) -> str:
        ret = '(' + datetime.fromtimestamp(t+FARCASTER_EPOCH).strftime('%Y-%m-%d %H:%M:%S') + ')'
        return colored( ret, 'light_grey')
    def pp_cast_id(self, fid:int, hash:str) -> str:
        out = colored(f"@{self.get_username_by_fid(fid)}", "magenta", attrs=["bold"])
        out += colored(f"/0x{hash} ", 'light_grey')
        return out
    def pp_link(self, l: str) -> str:
        return str(colored(l.strip(), "blue", attrs=['underline']))
    def pp_fid(self, fid: int) -> str:
        return colored(f"@{fid}", "magenta", attrs=["bold"])
    def pp_cast(self, m: Message, txt_offset=0 ) -> str:
        out = ""
        out +=  self.pp_cast_id(m.data.fid, m.hash.hex()) + self.pp_date(m.data.timestamp) + "\n"
        if m.data.cast_add_body.parent_cast_id.fid and not txt_offset:
            parent_fid = m.data.cast_add_body.parent_cast_id.fid
            parent_hash = m.data.cast_add_body.parent_cast_id.hash.hex()
            out = out + " "*txt_offset + f"↳ In reply to {self.pp_cast_id(parent_fid, parent_hash)}" + "\n"
        if m.data.cast_add_body.parent_url:
            out = out + " "*txt_offset + f"↳ In reply to {self.pp_link(m.data.cast_add_body.parent_url)}" + "\n"
        text = m.data.cast_add_body.text
        if hasattr(m.data.cast_add_body, 'mentions_positions'):
            offset = 0
            for i in range(len(m.data.cast_add_body.mentions)):
                pos = m.data.cast_add_body.mentions_positions[i]+offset
                username = f"{self.pp_fid(self.get_username_by_fid(m.data.cast_add_body.mentions[i]))}"
                text = text[0:pos] + username + text[pos:]
                offset += len(username)
        out += "\n".join([ "\n".join(textwrap.wrap(l, width=70)) for l in text.splitlines() ])
        if hasattr(m.data.cast_add_body, 'embeds'):
            for embed in m.data.cast_add_body.embeds:
                if hasattr(embed,'url'):
                    out += '\n> ' + self.pp_link(embed.url)
        lines = out.splitlines()
        if txt_offset:
            out2 = "  "*txt_offset + '┌ ' + lines[0]
        else:
            out2 = "  "*txt_offset + '┌ ' + lines[0]
        out2 += "\n"+"  "*txt_offset + '│ ' + ("\n"+"  "*txt_offset + '│ ').join(lines[1:])
        out2 += "\n" + "  "*txt_offset + '└─' 
        return out2

    def get_casts_by_fid(self, fid:int, count: int):
        ret = self.hub.GetCastsByFid(fid, count)
        messages = [m for m in ret.messages]
        messages.reverse()
        i=0
        for m in messages:
            i +=1
            print(self.pp_cast(m))
            if i > count:
                break
    def _get_cast_by_castid(self, fid:int, hash:str) -> Message:
        self.db_cur.execute("SELECT * FROM casts users WHERE fid=? AND hash=?", (fid,hash))
        cast = self.db_cur.fetchone()
        if cast:
            msg = Message()
            msg.ParseFromString(cast['msg'])
            return msg
        else:
            cast  = self.hub.GetCast(fid, hash)
            if not cast:
                return None
            self._db_put_cast(cast)
            return cast
    def _db_put_cast(self, cast: Message):
        ts = int(datetime.now().timestamp())
        if cast.data.cast_add_body.parent_cast_id.fid:
            parent_fid = cast.data.cast_add_body.parent_cast_id.fid
            parent_hash = '0x' + cast.data.cast_add_body.parent_cast_id.hash.hex()
            parent = f"{parent_fid}/{parent_hash}"
        elif cast.data.cast_add_body.parent_url:
            parent = cast.data.cast_add_body.parent_url
        else:
            parent = ''
        self.db_cur.execute("INSERT or REPLACE INTO casts(fid, hash, msg, parent, upd_ts) VALUES(?,?,?,?,?)", 
            (cast.data.fid, '0x'+cast.hash.hex(), cast.SerializeToString(), parent, ts)
        )
        self.db_conn.commit()

    def hdb_get_reactions_by_fid(self, fid:int, reaction_type: int, limit=100):
        username = self.get_username_by_fid(fid)
        out = ''
        page_token=1
        count=0
        while page_token:
            if page_token==1:
                page_token=None
            messages = self.hub.GetReactionsByFid(fid, reaction_type, 1000, page_token)
            page_token = messages.next_page_token
            #print(messages)
            heart = colored("❤", "red", attrs=["bold"])
            recast = colored("♺", "green", attrs=["bold"])
            for r in messages.messages:
                count += 1
                if limit and count >= limit:
                    break
                #print(r.data.reaction_body.type)
                if r.data.reaction_body.type == 1: #like
                    dt = datetime.fromtimestamp(r.data.timestamp+FARCASTER_EPOCH).strftime('%Y-%m-%d %H:%M:%S')
                    if r.data.reaction_body.target_cast_id.fid: # CastId
                        cast_id = self.pp_cast_id(
                            r.data.reaction_body.target_cast_id.fid,
                            r.data.reaction_body.target_cast_id.hash.hex()
                            )
                        desc = f"liked {cast_id}"
                    else:
                        desc = f"liked {r.data.reaction_body.target_url}"
                    out = f'{dt} {heart} {desc} \n' + out
                if r.data.reaction_body.type == 2: #recast
                    dt = datetime.fromtimestamp(r.data.timestamp+FARCASTER_EPOCH).strftime('%Y-%m-%d %H:%M:%S')
                    if r.data.reaction_body.target_cast_id.fid: # CastId
                        cast_id = self.pp_cast_id(
                            r.data.reaction_body.target_cast_id.fid,
                            r.data.reaction_body.target_cast_id.hash.hex()
                            )
                        desc = f"recasted {cast_id}"
                    else:
                        desc = f"recasted {r.data.reaction_body.target_url}"
                    out = f'{dt} {recast} {desc} \n' + out
                
                if r.data.reaction_body.type not in (1,2): #other
                    desc = "UNKNOWN reaction_type={r.data.reaction_body.type}"
                    out = f'{dt} {desc} \n' + out
            if limit and count >= limit:
                    break
        print('Count=', count)
        return out

    def hdb_get_reactions_by_cast(self, fid:int, oxhash:str, reaction_type: int):
        self.db_cur.execute("SELECT * FROM reactions WHERE fid=? AND hash=? AND reaction_type=? ORDER BY ts DESC", (fid, oxhash,1))
        r = self.db_cur.fetchone() 
        if r and (int(datetime.now().timestamp()) - int(r['ts']) < self.TTL):
            return self.db_cur.fetchall()
        else:            
            reactions = []
            page_token=1
            while page_token:
                if page_token==1:
                    page_token=None
                messages = self.hub.GetReactionsByCast(fid, oxhash, reaction_type, 100, page_token)
                page_token = messages.next_page_token
                for r in messages.messages:
                    u_fid = r.data.fid
                    ts = r.data.timestamp + FARCASTER_EPOCH
                    self.db_cur.execute(
                        "INSERT OR REPLACE INTO reactions(fid,hash, u_fid, reaction_type, ts) VALUES(?,?,?,?,?)", 
                        (fid, oxhash, u_fid, reaction_type, ts)
                    )
                    self.db_conn.commit()
            self.db_cur.execute(
                "SELECT * FROM reactions WHERE fid=? AND hash=? AND reaction_type=? ORDER BY ts DESC", 
                (fid, oxhash,1)
            )
            return self.db_cur.fetchall() 

    def h_get_cast_ancestors(self, fid:int, oxhash:str) -> {}:
        cast = self._get_cast_by_castid(fid, oxhash)
        parent_fid = cast.data.cast_add_body.parent_cast_id.fid
        while parent_fid:
            cast = self._get_cast_by_castid(parent_fid, '0x' + cast.data.cast_add_body.parent_cast_id.hash.hex() )
            parent_fid = cast.data.cast_add_body.parent_cast_id.fid
        return {"fid":cast.data.fid, "hash":'0x'+cast.hash.hex()}

    def h_get_cast_children(self, fid:int, oxhash:str):
        cast = self._get_cast_by_castid(fid, oxhash)
        children = self.hub.GetCastsByParentCast(fid, oxhash)
        c_list = [c for c in children.messages]
        for c in c_list:
            self._db_put_cast(c)
            self.h_get_cast_children( c.data.fid, '0x'+c.hash.hex() )

    def db_pp_cast_tree(self, fid:int, oxhash:str):
        top_cast = self.h_get_cast_ancestors(fid, oxhash)
        self.h_get_cast_children(top_cast['fid'], top_cast['hash'])
        return self.db_pp_cast_children(top_cast['fid'], top_cast['hash'])

    def db_pp_cast_children(self, fid:int, oxhash:str, offset=0, out=''):
        cast = self._get_cast_by_castid(fid, oxhash)
        out = ''
        children = self.db_cur.execute("SELECT * FROM casts WHERE parent=?", ( (f"{fid}/{oxhash}",) ))
        if children:
            rows = [row for row in children]
            for row in rows:
                c = Message()
                c.ParseFromString(row['msg'])
                out = out + self.db_pp_cast_children(c.data.fid, '0x'+c.hash.hex(), offset+1, out)
        out = self.pp_cast(cast, offset) + "\n" + out
        return out

    def db_pp_cast_children_orig(self, fid:int, oxhash:str, offset=0, out=''):
        cast = self._get_cast_by_castid(fid, oxhash)
        print(self.pp_cast(cast, offset))
        children = self.db_cur.execute("SELECT * FROM casts WHERE parent=?", ( (f"{fid}/{oxhash}",) ))
        if children:
            rows = [row for row in children]
            for row in rows:
                c = Message()
                c.ParseFromString(row['msg'])
                self.db_pp_cast_children(c.data.fid, '0x'+c.hash.hex(), offset+1, out)

    def get_cast(self, fid:int, hash: str, expand=False, append='', offset=0):
        ret = self._get_cast_by_castid(fid, hash)
        cast_txt = "\n" + self.pp_cast(ret, offset) + append
        if expand and ret.data.cast_add_body.parent_cast_id.fid:        
            cast_txt = self.get_cast(
                ret.data.cast_add_body.parent_cast_id.fid, '0x'+ret.data.cast_add_body.parent_cast_id.hash.hex(), True, cast_txt, offset
                )
        else:
            print(cast_txt)

def main():
    parser = argparse.ArgumentParser(prog="termcast", description="Terminal-based farcaster client")
    #parser.add_argument("--expand", help="Expand cast threads", action="store_true")
    parser.add_argument("--expand", type=int, default=0, help="Expand thread, level=<EXPAND>")
    parser.add_argument("--count", type=int, default=0, help="Limit number of resoults to <COUNT>")
    parser.add_argument("uri", type=str)
    args = parser.parse_args()
    
    p = Proxy()

    uri = re.findall('^@(\w*)(\.eth)?(/\w*)?(/\w*)?$', args.uri)
    if not uri:
        sys.exit(1)

    username = uri[0][0] + uri[0][1]
    path = uri[0][2]
    path2 = uri[0][3]

    # Look up fid if needed
    if not re.match("^\d*$", username):
        fid = p.get_fid_by_name(username)
    else:
        fid = int(username)

    USER_DATA_TYPE={
        'pfp':1,
        'display':2,
        'bio':3,
        'url':5,
        'username':6
    }
    if path[1:] in USER_DATA_TYPE:
        out = p.hdb_get_user_data(fid,USER_DATA_TYPE[path[1:]])
        print(out)
        sys.exit(0)
        # farcaster://<user>/(pfp|bio|url|username|casts|<link_type>)?
    if path[1:] == 'likes':
        out = p.hdb_get_reactions_by_fid(fid=fid, reaction_type=None, limit=args.count)
        print(out)
    if path == "/casts": 
        # List all casts
        p.get_casts_by_fid(fid, 20)
        sys.exit(0)
    elif path[0:3] == '/0x':
        if path2 not in ('/likes', '/recasts'):
            # Show cast
            if args.expand<2:
                p.get_cast(fid, path[1:], args.expand)
                print()
            else:
                print( p.db_pp_cast_tree(fid, path[1:]) )
                print()
                #from pick import pick
                #options = p.db_pp_cast_tree(fid, path[1:]).splitlines()
                #option, index = pick(options)
                #print(option)
                #print(index)
        elif path2 == '/likes':
            ret = p.hdb_get_reactions_by_cast(fid, path[1:], 1)
            for r in ret:
                username = p.get_username_by_fid(r['u_fid'])
                print(p.pp_fid(username))

if __name__ == '__main__':
    main()


import os
import sys
from dotenv import load_dotenv
import argparse
from . __about__ import version

def get_conf(required=[], args=None) -> str:

    CONFIG_FILE = os.path.expanduser('~/.fario')

    load_dotenv(CONFIG_FILE)
    load_dotenv()

    conf = {}
    params = ( 
        'hub', 'user_fid', 'user_key', 'app_fid', 'app_key', 'signer', 'op_eth_provider', 'ssl'
    )
    for p in params:
        conf[p] = os.getenv(p,'')
        if args and hasattr(args, p) and getattr(args, p):
            conf[p] = getattr(args,p)
    if len(required)>0 :
        check_conf(conf, required)
    return conf
def check_conf(conf, keys):
    for k in keys:
        if not conf[k]:
            print(f"Error: {k} is not set. Use the corresponding option, "
                    "an environment varaible, or set it in ~/.fario", 
                file=sys.stderr)
            sys.exit(1)
            
def cmd_conf_get(args):
    conf = get_conf()
    print("Configuration (using ~/.fario, .env and environment variables).")
    for k in conf:
        print(f"{k:<15} {conf[k]}")

def cmd_conf_make(args):
    conf = get_conf()
    print("# Set variables according to your configuration and save to ~/.fario.")
    print("# You can also leave them blank and set them using each command's options.")
    for k in conf:
        print(f"{k}={conf[k]}")

def cmd_conf():
    parser = argparse.ArgumentParser(prog='fario-config')
    parser.add_argument('--version', action='version', version='%(prog)s v'+version)
    subparser = parser.add_subparsers()

    cmd_config_get = subparser.add_parser("get", 
        description="Print current configuration. Priority, high-to-low: environment, .env, ~/.fario"
    )
    cmd_config_get.set_defaults(func=cmd_conf_get)

    cmd_config_make = subparser.add_parser("make", 
        description="Read configuration and output (stdout) a sample config file."
    )
    cmd_config_make.set_defaults(func=cmd_conf_make)

    args = parser.parse_args()

    if 'func' in args:
        args.func(args)

if __name__ == '__main__':
    conf = get_conf()
    for k in conf:
        print(f"{k:<15}: {conf[k]}")


#!/usr/bin/env python

import sys
import base64
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import MessageType, Message
from farcaster import FARCASTER_EPOCH
from google.protobuf.json_format import MessageToDict, MessageToJson
import argparse
from datetime import datetime
from termcolor import colored
import textwrap
from . config import get_conf
# from . __about__ import version

def pp_date(t: int) -> str:
	ret = '(' + datetime.fromtimestamp(t+FARCASTER_EPOCH).strftime('%Y-%m-%d %H:%M:%S') + ')'
	return colored( ret, 'light_grey')
def pp_link(l: str) -> str:
	return "\n> " + colored(l.strip(), "blue", attrs=['underline'])
def pp_cast_id(m: Message) -> str:
	out = colored(f"@{m.data.fid}", "green", attrs=["bold"])
	out += colored(f"/{m.hash.hex()} ", 'green')
	out += colored(
		datetime.fromtimestamp(m.data.timestamp+FARCASTER_EPOCH).strftime('%Y-%m-%d %H:%M:%S'),
		"light_grey"
		)
	return out
def pp_out(t: str) -> str:
	lines = t.split('\n')
	ret = '\n'.join([ "| "+l for l in lines ])
	return ret
def pp_fid(fid: int) -> str:
	return colored(f"@{fid}", "magenta", attrs=["bold"]) 

def pp_cast(m: Message, child_text='') -> str:
	# wrapper = textwrap.TextWrapper(width=70)
	assert m.data.type == MessageType.MESSAGE_TYPE_CAST_ADD, 'Not a CAST_ADD message'
	out = ""
	
	out += 	pp_cast_id(m) + "\n"

	text = m.data.cast_add_body.text
	if hasattr(m.data.cast_add_body, 'mentions_positions'):
		offset = 0
		for i in range(len(m.data.cast_add_body.mentions)):
			pos = m.data.cast_add_body.mentions_positions[i]+offset
			username = f"{pp_fid(m.data.cast_add_body.mentions[i])}"
			text = text[0:pos] + username + text[pos:]
			offset += len(username)
	out += "\n".join([ "\n".join(textwrap.wrap(l, width=70)) for l in text.splitlines() ])
	# out += "\n".join(textwrap.wrap(text, width=70))
	if hasattr(m.data.cast_add_body, 'embeds'):
		for embed in m.data.cast_add_body.embeds:
			if hasattr(embed,'url'):
				out += pp_link(embed.url)
	out = textwrap.indent(out, '| ')
	if m.data.cast_add_body.parent_cast_id.fid:
		out = f'↳ In reply to {m.data.cast_add_body.parent_cast_id.fid}/{m.data.cast_add_body.parent_cast_id.hash.hex()}\n' + out
	return out

def main():
	parser = argparse.ArgumentParser(prog="fario-pprint", description="Convert fario export to human-friendly output")
	#parser.add_argument("--lines", type=int, help="Only parse first <LINES>", default=0)
	parser.add_argument("--limit", type=int, help="Number of records. If more than one types of data are exported, the limit applies to each one separately.", default=sys.maxsize)
	# parser.add_argument('--version', action='version', version='%(prog)s v'+version)
	parser.add_argument("fid", type=int, help="Export messages from fid=FID")
	args = parser.parse_args()

	conf = get_conf(required=['hub'], args=args)
	hub = HubService(conf['hub'], use_async=False)
	ret = hub.GetCastsByFid(args.fid, 10)

	count=0
	for m in ret.messages:
		#m = Message.FromString(base64.b64decode(line))
		if m.data.type == MessageType.MESSAGE_TYPE_CAST_ADD:
			out = pp_cast(m)
		else:
			out = None
		
		if out:
			print()
			print(out)

		if args.limit and count >= args.limit:
			break

if __name__ == '__main__':
	main()


#!/usr/bin/env python

import sys
import base64
from farcaster.fcproto.message_pb2 import MessageType, Message
from google.protobuf.json_format import MessageToDict, MessageToJson
import argparse
from . __about__ import version

def main():
	parser = argparse.ArgumentParser(prog="fario_grep", 
		description="A utility to read fario records from stdin, and select the ones that match a given string.")
	parser.add_argument("--signer", action='store_true', help="Treat search string as signer public key. Must be in hex starting with '0x'", default=0)
	parser.add_argument("--not-signer", action='store_true', help="Treat search string as signer public key. Match !search_string. Must be in hex starting with '0x'", default=0)
	parser.add_argument("search_string", type=str, help="String to search for")
	parser.add_argument('--version', action='version', version='%(prog)s v'+version)
	args = parser.parse_args()

	for line in sys.stdin:
		m = Message.FromString(base64.b64decode(line))
		if args.signer:
			if m.signer==bytes.fromhex(args.search_string[2:]):
				print(line.strip())
		elif args.not_signer:
			if m.signer!=bytes.fromhex(args.search_string[2:]):
				print(line.strip())
		elif str(m).find(args.search_string) >-1 :
			print(line)

if __name__ == '__main__':
	main()


#!/usr/bin/env python

import sys
import base64
from farcaster.fcproto.message_pb2 import MessageType, Message
from google.protobuf.json_format import MessageToDict, MessageToJson
import argparse
import json

def main():
	parser = argparse.ArgumentParser(prog="fario2json", description="Convert fario export to json")
	parser.add_argument("--lines", type=int, help="Only parse first <LINES>", default=0)
	args = parser.parse_args()

	print("[")
	separator = ''
	count=0
	for line in sys.stdin:
		m = Message.FromString(base64.b64decode(line))

		out = MessageToJson(m)
		out = json.loads(out)
		out['signer'] = base64.b64decode(out['signer']).hex()
		out['hash'] = base64.b64decode(out['hash']).hex()
		out = json.dumps(out)
		print(separator+str(out))
		if not separator:
			separator = ', '
		if args.lines and count >= args.lines:
			break
			
	print("]")

if __name__ == '__main__':
	main()


#!/usr/bin/env python
import os
import sys
from time import sleep
from dotenv import load_dotenv
from farcaster.HubService import HubService
from farcaster.fcproto.message_pb2 import MessageType, Message
import base64
import argparse
from . config import get_conf
from . __about__ import version


def get_all_messages(method, fid, page_size, wait):
	page_token="1strun"
	while page_token:
		if page_token == "1strun":
			page_token = None
		response = method(fid=fid, page_size=page_size, page_token=page_token)
		for message in response.messages:
			out = base64.b64encode(message.SerializeToString()).decode('ascii')
			yield out
		page_token = response.next_page_token
		sleep(wait)

def main():		
	parser = argparse.ArgumentParser(prog="fario-backup", description="Export Farcaster data.")
	parser.add_argument('--version', action='version', version='%(prog)s v'+version)
	parser.add_argument("fid", type=int, help="Export messages from fid=FID")
	parser.add_argument("--hub", help="Use the hub at <HUB>. Ex. --hub 192.168.1.1:2283", type=str)
	parser.add_argument("--ssl", help="Use SSL", action="store_true")
	parser.add_argument("--wait", type=int, help="Wait for <WAIT> milliseconds between reads.", default=0)
	args = parser.parse_args()

	conf = get_conf(required=['hub'], args=args)

	hub = HubService(conf['hub'], use_async=False, use_ssl=conf['ssl'])

	for c in get_all_messages(hub.GetAllCastMessagesByFid, args.fid, 1000, args.wait):
		print(c)
	#for c in get_data(hub.GetLinksByFid, args.fid, 100, args.limit, args.wait):
	#	print(c)
	# for c in get_reactions(hub.GetReactionsByFid, args.fid, None, 1000, args.wait):
	#	print(c)
	#for c in get_data(hub.GetUserDataByFid, args.fid, 100, args.limit, args.wait):
	#	print(c)

if __name__ == '__main__':
	main()